#include <proto.h>

#include "gtscrlist.h"
#include "gtgroup.h"
#include "gtscrollbar.h"
#include "gtlist.h"
#include "gtapp.h"
#include "gterror.h"
#include "gtresname.h"
#include "gtcommon.h"

/************************************************************************\
*
* GtScrList class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/************************************************************************\
*
* Create and init scrlist.
*
\************************************************************************/
GtScrList *GtMkScrList(char *n,GtManager *p,GtTag *tgs)
{
	GtScrList *obj;
	
	if(!(obj=new GtScrList()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtScrList object!\n");
		return(NULL);
	}
	if(!obj->Init(n,p,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init scrlist.
*
\************************************************************************/
GtScrList *GtMkScrList(char *n,GtManager *p,ulong tag,...)
{
	va_list va;
	GtScrList *obj;
	int ps;
	
	va_start(va,tag);
	ps=GtBeginVaTags();
	obj=GtMkScrList(n,p,GtCopyVaTags(tag,&va));
	GtEndVaTags(ps);
	va_end(va);
	return(obj);
}

/************************************************************************\
*
* Setup empty scrlist.
*
\************************************************************************/
GtScrList::GtScrList(void)
{
	grp=NULL;
	sg=NULL;
	list=NULL;
	scr=NULL;
}

/************************************************************************\
*
* Destroy scrlist.
*
\************************************************************************/
void GtScrList::Destroy(void)
{
	if(grp)
		GtDestroy(grp);
	GtWidget::Destroy();
}

/************************************************************************\
*
* Setup scrlist.
*
\************************************************************************/
sbool GtScrList::Init(char *n,GtManager *p,GtTag *tgs)
{
	GtTag *t;
	ulong sz=10;
	long idel=250,del=100;
	ubyte ap=GTAP_TOGETHER;
	
	if(!GtWidget::Init(n,p,tgs))
		return(FALSE);
	app->GetRes(this,GTR_SBSZ,GTRT_LONG,&sz);
	app->GetRes(this,GTR_APLACE,GTRT_APLACE,&ap);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_ScrBarSize:
				sz=t->d.val;
				break;
			case GT_ArrowPlace:
				ap=t->d.val;
				if(ap>GTAP_LAST_NR)
					ap=GTAP_TOGETHER;
				break;
			case GT_InitialDelay:
				idel=t->d.val;
				break;
			case GT_Delay:
				del=t->d.val;
				break;
		}
	}
	if(!(grp=GtMkGroup("slgroup",p,GT_Orient,GTO_HORIZ,GT_DONE)))
		return(FALSE);
	if(!(list=GtMkList("list",grp,NULL)))
		return(FALSE);
	list->AddCB(GT_ValueChange,ListChange,this);
	if(!(scr=GtMkScrollBar("sbar",grp,
			GT_Orient,GTO_VERT,
			GT_ScrBarSize,sz,
			GT_ArrowPlace,ap,
			GT_InitialDelay,idel,
			GT_Delay,del,
			GT_Start,0,
			GT_End,0,
			GT_ReportDrag,TRUE,
			GT_ReduceInterval,TRUE,
			GT_DONE)))
		return(FALSE);
	scr->GetScroller()->AddCB(GT_ValueChange,ScrChange,this);
	return(TRUE);
}

/************************************************************************\
*
* Set scrlist attributes.
*
\************************************************************************/
sbool GtScrList::Set(GtTag *tgs)
{
	GtTag *t;
	ulong sz=0;
	long idel=0,del=0;
	
	if(!GtWidget::Set(tgs))
		return(FALSE);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_ScrBarSize:
				sz=t->d.val;
				break;
			case GT_InitialDelay:
				idel=t->d.val;
				break;
			case GT_Delay:
				del=t->d.val;
				break;
		}
	}
	if(idel || del)
	{
		scr->SetV(
			(idel ? GT_InitialDelay : GT_IGNORE),idel,
			(del ? GT_Delay : GT_IGNORE),del,
			GT_DONE);
	}
	if(sz)
		scr->SetV(GT_MinXSize,sz,GT_DONE);
	return(TRUE);
}

/************************************************************************\
*
* Get scrlist attributes.
*
\************************************************************************/
void GtScrList::Get(GtTag *tgs)
{
	GtTag *t;
	long *ptr;
	
	GtWidget::Get(tgs);
	while((t=GtNextTag(&tgs)))
	{
		if(!(ptr=(long *)t->d.ptr))
			continue;
		switch(t->tag)
		{
			case GT_ScrBarSize:
				*ptr=scr->GetA(GT_MinXSize,0);
				break;
			case GT_InitialDelay:
				*ptr=scr->GetA(GT_InitialDelay,0);
				break;
			case GT_Delay:
				*ptr=scr->GetA(GT_Delay,0);
				break;
		}
	}
}

/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtScrList::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtScrList")==0)
		return(TRUE);
	else
		return(GtWidget::IsSubClassOf(cn));
}

/************************************************************************\
*
* List values changed callback.
*
\************************************************************************/
int GtScrList::ListChange(GtBase *obj,GtCDPtr cd,GtUDPtr ud)
{
	GtScrList *sl;
	GtListChangeCD *lcd;

	obj=obj;
	lcd=(GtListChangeCD *)cd;
	sl=(GtScrList *)ud;
	sl->scr->SetV(GT_Value,lcd->top,
								GT_Page,lcd->visible,
								GT_End,lcd->total,
								GT_DONE);
	return(GTCB_CONT);
}

/************************************************************************\
*
* Scroller value changed callback.
*
\************************************************************************/
int GtScrList::ScrChange(GtBase *obj,GtCDPtr cd,GtUDPtr ud)
{
	GtScrList *sl;
	GtScrollerCD *scd;
	
	obj=obj;
	scd=(GtScrollerCD *)cd;
	sl=(GtScrList *)ud;
	sl->list->SetV(GT_Top,scd->value,GT_DONE);
	return(GTCB_CONT);
}
